@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,23 +11,27 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::BulkWriteView;
 
 # ABSTRACT: Bulk write operations against a query document
 
 use version;
-our $VERSION = 'v1.2.2';
+our $VERSION = 'v2.2.0';
 
 use Moo;
 
 use MongoDB::Error;
 use MongoDB::_Types qw(
+    Boolish
+    Document
     IxHash
-    Booleanpm
 );
 use Types::Standard qw(
+    Maybe
+    ArrayRef
     InstanceOf
 );
 use boolean;
@@ -50,12 +53,32 @@ has _bulk => (
     handles  => [qw/_enqueue_write/]
 );
 
+has _collation => (
+    is  => 'ro',
+    isa => Maybe [Document],
+);
+
+has _array_filters => (
+    is => 'ro',
+    isa => Maybe [ArrayRef[Document]],
+);
+
 has _upsert => (
     is      => 'ro',
-    isa     => Booleanpm,
-    default => sub { false },
+    isa     => Boolish,
+    default => 0,
 );
 
+sub collation {
+    my ($self, $collation) = @_;
+    return $self->new( %$self, _collation => $collation );
+}
+
+sub arrayFilters {
+  my ( $self, $array_filters ) = @_;
+  return $self->new( %$self, _array_filters => $array_filters );
+}
+
 sub upsert {
     my ($self) = @_;
     unless ( @_ == 1 ) {
@@ -97,12 +120,16 @@ sub _update {
         $doc = Tie::IxHash->new(%$doc);
     }
 
+    $self->_bulk->_retryable( 0 ) if $method eq 'update_many';
+
     my $update = {
         q      => $self->_query,
         u      => $doc,
         multi  => $method eq 'update_many' ? true : false,
         upsert => boolean( $self->_upsert ),
         is_replace => $method eq 'replace_one',
+        (defined $self->_collation ? (collation => $self->_collation) : ()),
+        (defined $self->_array_filters ? (arrayFilters => $self->_array_filters) : ()),
     };
 
     $self->_enqueue_write( [ update => $update ] );
@@ -112,28 +139,35 @@ sub _update {
 
 sub delete_many {
     my ($self) = @_;
-    $self->_enqueue_write( [ delete => { q => $self->_query, limit => 0 } ] );
+    $self->_bulk->_retryable( 0 );
+    $self->_enqueue_write(
+        [
+            delete => {
+                q     => $self->_query,
+                limit => 0,
+                ( defined $self->_collation ? ( collation => $self->_collation ) : () ),
+                (defined $self->_array_filters ? (arrayFilters => $self->_array_filters) : ()),
+            }
+        ]
+    );
     return;
 }
 
 sub delete_one {
     my ($self) = @_;
-    $self->_enqueue_write( [ delete => { q => $self->_query, limit => 1 } ] );
+    $self->_enqueue_write(
+        [
+            delete => {
+                q     => $self->_query,
+                limit => 1,
+                ( defined $self->_collation ? ( collation => $self->_collation ) : () ),
+                (defined $self->_array_filters ? (arrayFilters => $self->_array_filters) : ()),
+            }
+        ]
+    );
     return;
 }
 
-#--------------------------------------------------------------------------#
-# Deprecated methods
-#--------------------------------------------------------------------------#
-
-BEGIN {
-    no warnings 'once';
-    *update = \&update_many;
-    *remove = \&delete_many;
-    *remove_one = \&delete_one;
-}
-
-
 1;
 
 __END__
@@ -148,7 +182,7 @@ MongoDB::BulkWriteView - Bulk write operations against a query document
 
 =head1 VERSION
 
-version v1.2.2
+version v2.2.0
 
 =head1 SYNOPSIS
 
@@ -160,6 +194,10 @@ version v1.2.2
     # Update all documents matching the selector
     bulk->find( { a => 2 } )->update_many( { '$inc' => { x => 2 } } );
 
+    # Update all documents matching the selector, with respect to a collation
+    bulk->find( { a => { '$gte' => 'F' } )->collation($collation)
+          ->update_many( { '$inc' => { x => 2 } } );
+
     # Update all documents
     bulk->find( {} )->update_many( { '$inc' => { x => 2 } } );
 
@@ -181,6 +219,12 @@ version v1.2.2
     # Remove all documents matching the selector
     bulk->find( { a => 5 } )->delete_many();
 
+    # Update any arrays with the matching filter
+    bulk->find( {} )->arrayFilters([ { 'i.b' => 1 } ])->update_many( { '$set' => { 'y.$[i].b' => 2 } } );
+
+    # Remove all documents matching the selector, with respect to a collation
+    bulk->find( { a => { '$gte' => 'F' } )->collation($collation)->delete_many();
+
     # Remove all documents
     bulk->find( {} )->delete_many();
 
@@ -190,13 +234,30 @@ This class provides means to specify write operations constrained by a query
 document.
 
 To instantiate a C<MongoDB::BulkWriteView>, use the L<find|MongoDB::BulkWrite/find>
-method from L<MongoDB::BulkWrite> or the L</upsert> method described below.
+method from L<MongoDB::BulkWrite>.
 
-Except for L</upsert>, all methods have an empty return on success; an
-exception will be thrown on error.
+Except for L</arrayFilters>, L</collation> and L</upsert>, all methods have an
+empty return on success; an exception will be thrown on error.
 
 =head1 METHODS
 
+=head2 arrayFilters
+
+    $bulk->arrayFilters( $array_filters )->update_many( $modification );
+
+Returns a new C<MongoDB::BulkWriteView> object, where the specified arrayFilter
+will be used to determine which array elements to modify for an update
+operation on an array field.
+
+=head2 collation
+
+    $bulk->collation( $collation )->delete_one;
+
+Returns a new C<MongoDB::BulkWriteView> object, where the specified
+collation will be used to determine which documents match the query
+document.  A collation can be specified for any deletion, replacement,
+or update.
+
 =head2 delete_many
 
     $bulk->delete_many;
@@ -247,6 +308,10 @@ David Golden <david@mongodb.com>
 
 =item *
 
+Rassi <rassi@mongodb.com>
+
+=item *
+
 Mike Friedman <friedo@friedo.com>
 
 =item *
@@ -261,7 +326,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc..
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
