@@ -1,5 +1,4 @@
-#
-#  Copyright 2009-2013 MongoDB, Inc.
+#  Copyright 2009 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,28 +11,31 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Database;
 
 
 # ABSTRACT: A MongoDB Database
 
 use version;
-our $VERSION = 'v1.2.2';
+our $VERSION = 'v2.2.0';
 
 use MongoDB::CommandResult;
 use MongoDB::Error;
-use MongoDB::GridFS;
+use MongoDB::GridFSBucket;
+use MongoDB::Op::_Command;
+use MongoDB::Op::_DropDatabase;
 use MongoDB::Op::_ListCollections;
 use MongoDB::ReadPreference;
-use MongoDB::_Query;
 use MongoDB::_Types qw(
     BSONCodec
     NonNegNum
     ReadPreference
     ReadConcern
     WriteConcern
+    is_OrderedDoc
 );
 use Types::Standard qw(
     InstanceOf
@@ -42,7 +44,6 @@ use Types::Standard qw(
 use Carp 'carp';
 use boolean;
 use Moo;
-use Try::Tiny;
 use namespace::clean -except => 'meta';
 
 has _client => (
@@ -130,8 +131,8 @@ has max_time_ms => (
 #pod =attr bson_codec
 #pod
 #pod An object that provides the C<encode_one> and C<decode_one> methods, such as
-#pod from L<MongoDB::BSON>.  It may be initialized with a hash reference that will
-#pod be coerced into a new MongoDB::BSON object.  By default it will be inherited
+#pod from L<BSON>.  It may be initialized with a hash reference that will
+#pod be coerced into a new L<BSON> object.  By default it will be inherited
 #pod from a L<MongoDB::MongoClient> object.
 #pod
 #pod =cut
@@ -147,6 +148,20 @@ has bson_codec => (
 # methods
 #--------------------------------------------------------------------------#
 
+#pod =method client
+#pod
+#pod     $client = $db->client;
+#pod
+#pod Returns the L<MongoDB::MongoClient> object associated with this
+#pod object.
+#pod
+#pod =cut
+
+sub client {
+    my ($self) = @_;
+    return $self->_client;
+}
+
 #pod =method list_collections
 #pod
 #pod     $result = $coll->list_collections( $filter );
@@ -177,11 +192,16 @@ has bson_codec => (
 #pod * C<batchSize> – the number of documents to return per batch.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
+#pod * C<nameOnly> - query and return names of the collections only. Defaults to
+#pod   false. (Note, this will be ignored for servers before version 4.0)
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
+#pod
+#pod B<NOTE>: When using C<nameOnly>, the filter query must be empty or must only
+#pod query the C<name> field or else no documents will be found.
 #pod
 #pod =cut
 
-my $list_collections_args;
-
 sub list_collections {
     my ( $self, $filter, $options ) = @_;
     $filter  ||= {};
@@ -192,41 +212,61 @@ sub list_collections {
         $options->{maxTimeMS} = $self->max_time_ms;
     }
 
+    my $session = $self->_client->_get_session_from_hashref( $options );
+
     my $op = MongoDB::Op::_ListCollections->_new(
-        db_name    => $self->name,
-        client     => $self->_client,
-        bson_codec => $self->bson_codec,
-        filter     => $filter,
-        options    => $options,
+        db_name             => $self->name,
+        client              => $self->_client,
+        bson_codec          => $self->bson_codec,
+        filter              => $filter,
+        options             => $options,
+        session             => $session,
+        monitoring_callback => $self->_client->monitoring_callback,
+        read_preference     => MongoDB::ReadPreference->new( mode => 'primary' ),
     );
 
-    return $self->_client->send_read_op($op);
+    return $self->_client->send_retryable_read_op($op);
 }
 
 #pod =method collection_names
 #pod
 #pod     my @collections = $database->collection_names;
+#pod     my @collections = $database->collection_names( $filter );
+#pod     my @collections = $database->collection_names( $filter, $options );
 #pod
 #pod Returns the list of collections in this database.
 #pod
-#pod B<Warning:> if the number of collections is very large, this will return
-#pod a very large result.  Use L</list_collections> to iterate over collections
-#pod instead.
+#pod An optional filter document may be provided, which cause only collection
+#pod description documents matching a filter expression to be returned.  See the
+#pod L<listCollections command
+#pod documentation|http://docs.mongodb.org/manual/reference/command/listCollections/>
+#pod for more details on filtering for specific collections.
+#pod
+#pod A hashref of options may also be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
+#pod
+#pod B<Warning:> if the number of collections is very large, this may return
+#pod a very large result.  Either pass an appropriate filter, or use
+#pod L</list_collections> to iterate over collections instead.
 #pod
 #pod =cut
 
 sub collection_names {
-    my ($self) = @_;
+    my ( $self, $filter, $options ) = @_;
 
-    my $op = MongoDB::Op::_ListCollections->_new(
-        db_name    => $self->name,
-        client     => $self->_client,
-        bson_codec => $self->bson_codec,
-        filter     => {},
-        options    => {},
-    );
+    $filter ||= {};
+    $options ||= {};
 
-    my $res = $self->_client->send_read_op($op);
+    my @filter_keys = keys %$filter;
+    $options->{nameOnly} =
+      @filter_keys == 0 || ( @filter_keys == 1 && exists $filter->{name} );
+
+    my $res = $self->list_collections( $filter, $options );
 
     return map { $_->{name} } $res->all;
 }
@@ -264,50 +304,72 @@ sub get_collection {
 
 { no warnings 'once'; *coll = \&get_collection }
 
-#pod =method get_gridfs
+#pod =method get_gridfsbucket, gfs
 #pod
-#pod     my $grid = $database->get_gridfs;
-#pod     my $grid = $database->get_gridfs("fs");
-#pod     my $grid = $database->get_gridfs("fs", $options);
+#pod     my $grid = $database->get_gridfsbucket;
+#pod     my $grid = $database->get_gridfsbucket($options);
+#pod     my $grid = $database->gfs($options);
 #pod
-#pod Returns a L<MongoDB::GridFS> for storing and retrieving files from the database.
-#pod Default prefix is "fs", making C<$grid-E<gt>files> "fs.files" and C<$grid-E<gt>chunks>
-#pod "fs.chunks".
+#pod This method returns a L<MongoDB::GridFSBucket> object for storing and
+#pod retrieving files from the database.
 #pod
 #pod It takes an optional hash reference of options that are passed to the
-#pod L<MongoDB::GridFS> constructor.
+#pod L<MongoDB::GridFSBucket> constructor.
 #pod
-#pod See L<MongoDB::GridFS> for more information.
+#pod See L<MongoDB::GridFSBucket> for more information.
+#pod
+#pod The C<gfs> method is an alias for C<get_gridfsbucket>.
 #pod
 #pod =cut
 
-sub get_gridfs {
-    my ($self, $prefix, $options) = @_;
-    $prefix = "fs" unless $prefix;
+sub get_gridfsbucket {
+    my ($self, $options) = @_;
 
-    return MongoDB::GridFS->new(
+    return MongoDB::GridFSBucket->new(
         read_preference => $self->read_preference,
         write_concern   => $self->write_concern,
-        max_time_ms     => $self->max_time_ms,
+        read_concern    => $self->read_concern,
         bson_codec      => $self->bson_codec,
+        max_time_ms     => $self->max_time_ms,
         ( $options ? %$options : () ),
         # not allowed to be overridden by options
-        _database => $self,
-        prefix => $prefix
-    );
+        database => $self,
+    )
 }
 
+{ no warnings 'once'; *gfs = \&get_gridfsbucket }
+
 #pod =method drop
 #pod
 #pod     $database->drop;
 #pod
 #pod Deletes the database.
 #pod
+#pod A hashref of options may also be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
+#pod
 #pod =cut
 
 sub drop {
-    my ($self) = @_;
-    return $self->run_command({ 'dropDatabase' => 1 });
+    my ( $self, $options ) = @_;
+
+    my $session = $self->_client->_get_session_from_hashref( $options );
+
+    return $self->_client->send_write_op(
+        MongoDB::Op::_DropDatabase->_new(
+            client              => $self->_client,
+            db_name             => $self->name,
+            bson_codec          => $self->bson_codec,
+            write_concern       => $self->write_concern,
+            session             => $session,
+            monitoring_callback => $self->_client->monitoring_callback,
+        )
+    )->output;
 }
 
 #pod =method run_command
@@ -319,17 +381,31 @@ sub drop {
 #pod         { mode => 'secondaryPreferred' }
 #pod     );
 #pod
+#pod     my $output = $database->run_command(
+#pod         [ some_command => 1 ],
+#pod         $read_preference,
+#pod         $options
+#pod     );
+#pod
 #pod This method runs a database command.  The first argument must be a document
 #pod with the command and its arguments.  It should be given as an array reference
 #pod of key-value pairs or a L<Tie::IxHash> object with the command name as the
-#pod first key.  The use of a hash reference will only reliably work for commands
-#pod without additional parameters.
+#pod first key.  An error will be thrown if the command is not an
+#pod L<ordered document|MongoDB::Collection/Ordered document>.
 #pod
 #pod By default, commands are run with a read preference of 'primary'.  An optional
 #pod second argument may specify an alternative read preference.  If given, it must
 #pod be a L<MongoDB::ReadPreference> object or a hash reference that can be used to
 #pod construct one.
 #pod
+#pod A hashref of options may also be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
+#pod
 #pod It returns the output of the command (a hash reference) on success or throws a
 #pod L<MongoDB::DatabaseError|MongoDB::Error/MongoDB::DatabaseError> exception if
 #pod the command fails.
@@ -345,53 +421,169 @@ sub drop {
 #pod =cut
 
 sub run_command {
-    my ( $self, $command, $read_pref ) = @_;
+    my ( $self, $command, $read_pref, $options ) = @_;
+    MongoDB::UsageError->throw("command was not an ordered document")
+       if ! is_OrderedDoc($command);
 
     $read_pref = MongoDB::ReadPreference->new(
         ref($read_pref) ? $read_pref : ( mode => $read_pref ) )
       if $read_pref && ref($read_pref) ne 'MongoDB::ReadPreference';
 
+    my $session = $self->_client->_get_session_from_hashref( $options );
+
     my $op = MongoDB::Op::_Command->_new(
-        db_name     => $self->name,
-        query       => $command,
-        query_flags => {},
-        bson_codec  => $self->bson_codec,
-        read_preference => $read_pref,
+        client              => $self->_client,
+        db_name             => $self->name,
+        query               => $command,
+        query_flags         => {},
+        bson_codec          => $self->bson_codec,
+        read_preference     => $read_pref,
+        session             => $session,
+        monitoring_callback => $self->_client->monitoring_callback,
     );
 
-    my $obj = $self->_client->send_read_op($op);
+    my $obj = $self->_client->send_retryable_read_op($op);
 
     return $obj->output;
 }
 
-#--------------------------------------------------------------------------#
-# deprecated methods
-#--------------------------------------------------------------------------#
+#pod =method aggregate
+#pod
+#pod Runs a query using the MongoDB 3.6+ aggregation framework and returns a
+#pod L<MongoDB::QueryResult> object.
+#pod
+#pod The first argument must be an array-ref of L<aggregation
+#pod pipeline|http://docs.mongodb.org/manual/core/aggregation-pipeline/> documents.
+#pod Each pipeline document must be a hash reference.
+#pod
+#pod The server supports several collection-less aggregation source stages like
+#pod C<$currentOp> and C<$listLocalSessions>.
+#pod
+#pod     $result = $database->aggregate( [
+#pod         {
+#pod             "\$currentOp" => {
+#pod                 allUsers => true,
+#pod             },
+#pod         },
+#pod     ] );
+#pod
+#pod See L<Aggregation|http://docs.mongodb.org/manual/aggregation/> in the MongoDB manual
+#pod for more information on how to construct aggregation queries.
+#pod
+#pod =cut
 
-sub eval {
-    my ($self, $code, $args, $nolock) = @_;
+sub aggregate {
+    MongoDB::UsageError->throw("pipeline argument must be an array reference")
+      unless ref( $_[1] ) eq 'ARRAY';
 
-    $nolock = boolean::false unless defined $nolock;
+    my ( $self, $pipeline, $options ) = @_;
+    $options ||= {};
 
-    my $cmd = tie(my %hash, 'Tie::IxHash');
-    %hash = ('$eval' => $code,
-             'args' => $args,
-             'nolock' => $nolock);
+    my $session = $self->_client->_get_session_from_hashref( $options );
 
-    my $output = $self->run_command($cmd);
-    if (ref $output eq 'HASH' && exists $output->{'retval'}) {
-        return $output->{'retval'};
+    # boolify some options
+    for my $k (qw/allowDiskUse explain/) {
+        $options->{$k} = ( $options->{$k} ? true : false ) if exists $options->{$k};
     }
-    else {
-        return $output;
+
+    # possibly fallback to default maxTimeMS
+    if ( ! exists $options->{maxTimeMS} && $self->max_time_ms ) {
+        $options->{maxTimeMS} = $self->max_time_ms;
     }
+
+    # read preferences are ignored if the last stage is $out or $merge
+    my ($last_op) = keys %{ $pipeline->[-1] };
+
+    my $op = MongoDB::Op::_Aggregate->_new(
+        pipeline        => $pipeline,
+        options         => $options,
+        read_concern    => $self->read_concern,
+        has_out         => !!($last_op =~ m/\$out|\$merge/),
+        client          => $self->_client,
+        bson_codec      => $self->bson_codec,
+        db_name         => $self->name,
+        coll_name       => 1,                     # Magic not-an-actual-collection number
+        full_name       => $self->name . ".1",
+        read_preference => $self->read_preference,
+        write_concern   => $self->write_concern,
+        session         => $session,
+        monitoring_callback => $self->_client->monitoring_callback,
+    );
+
+    return $self->_client->send_retryable_read_op($op);
 }
 
-sub last_error {
-    my ( $self, $opt ) = @_;
-    return $self->run_command( [ getlasterror => 1, ( $opt ? %$opt : () ) ] );
-}
+#pod =method watch
+#pod
+#pod Watches for changes on this database.
+#pod
+#pod Perform an aggregation with an implicit initial C<$changeStream> stage
+#pod and returns a L<MongoDB::ChangeStream> result which can be used to
+#pod iterate over the changes in the database. This functionality is
+#pod available since MongoDB 4.0.
+#pod
+#pod     my $stream = $db->watch();
+#pod     my $stream = $db->watch( \@pipeline );
+#pod     my $stream = $db->watch( \@pipeline, \%options );
+#pod
+#pod     while (1) {
+#pod
+#pod         # This inner loop will only run until no more changes are
+#pod         # available.
+#pod         while (my $change = $stream->next) {
+#pod             # process $change
+#pod         }
+#pod     }
+#pod
+#pod The returned stream will not block forever waiting for changes. If you
+#pod want to respond to changes over a longer time use C<maxAwaitTimeMS> and
+#pod regularly call C<next> in a loop.
+#pod
+#pod See L<MongoDB::Collection/watch> for details on usage and available
+#pod options.
+#pod
+#pod =cut
 
+sub watch {
+    my ( $self, $pipeline, $options ) = @_;
+
+    $pipeline ||= [];
+    $options ||= {};
+
+    my $session = $self->_client->_get_session_from_hashref( $options );
+
+    return MongoDB::ChangeStream->new(
+        exists($options->{startAtOperationTime})
+            ? (start_at_operation_time => delete $options->{startAtOperationTime})
+            : (),
+        exists($options->{fullDocument})
+            ? (full_document => delete $options->{fullDocument})
+            : (full_document => 'default'),
+        exists($options->{resumeAfter})
+            ? (resume_after => delete $options->{resumeAfter})
+            : (),
+        exists($options->{startAfter})
+            ? (start_after => delete $options->{startAfter})
+            : (),
+        exists($options->{maxAwaitTimeMS})
+            ? (max_await_time_ms => delete $options->{maxAwaitTimeMS})
+            : (),
+        client => $self->_client,
+        pipeline => $pipeline,
+        session => $session,
+        options => $options,
+        op_args => {
+            read_concern => $self->read_concern,
+            bson_codec => $self->bson_codec,
+            db_name => $self->name,
+            coll_name => 1,                     # Magic not-an-actual-collection number
+            full_name => $self->name . ".1",
+            read_preference => $self->read_preference,
+            write_concern => $self->write_concern,
+            monitoring_callback => $self->_client->monitoring_callback,
+        },
+    );
+}
 
 1;
 
@@ -407,7 +599,7 @@ MongoDB::Database - A MongoDB Database
 
 =head1 VERSION
 
-version v1.2.2
+version v2.2.0
 
 =head1 SYNOPSIS
 
@@ -433,7 +625,7 @@ call C<get_database> on a L<MongoDB::MongoClient> object.
 
 =head2 Error handling
 
-Unless otherwise explictly documented, all methods throw exceptions if
+Unless otherwise explicitly documented, all methods throw exceptions if
 an error occurs.  The error types are documented in L<MongoDB::Error>.
 
 To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
@@ -494,12 +686,19 @@ was when the C<$maxTimeMS> meta-operator was introduced.
 =head2 bson_codec
 
 An object that provides the C<encode_one> and C<decode_one> methods, such as
-from L<MongoDB::BSON>.  It may be initialized with a hash reference that will
-be coerced into a new MongoDB::BSON object.  By default it will be inherited
+from L<BSON>.  It may be initialized with a hash reference that will
+be coerced into a new L<BSON> object.  By default it will be inherited
 from a L<MongoDB::MongoClient> object.
 
 =head1 METHODS
 
+=head2 client
+
+    $client = $db->client;
+
+Returns the L<MongoDB::MongoClient> object associated with this
+object.
+
 =head2 list_collections
 
     $result = $coll->list_collections( $filter );
@@ -536,17 +735,48 @@ C<batchSize> – the number of documents to return per batch.
 
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 2.6.)
 
+=item *
+
+C<nameOnly> - query and return names of the collections only. Defaults to false. (Note, this will be ignored for servers before version 4.0)
+
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
 =back
 
+B<NOTE>: When using C<nameOnly>, the filter query must be empty or must only
+query the C<name> field or else no documents will be found.
+
 =head2 collection_names
 
     my @collections = $database->collection_names;
+    my @collections = $database->collection_names( $filter );
+    my @collections = $database->collection_names( $filter, $options );
 
 Returns the list of collections in this database.
 
-B<Warning:> if the number of collections is very large, this will return
-a very large result.  Use L</list_collections> to iterate over collections
-instead.
+An optional filter document may be provided, which cause only collection
+description documents matching a filter expression to be returned.  See the
+L<listCollections command
+documentation|http://docs.mongodb.org/manual/reference/command/listCollections/>
+for more details on filtering for specific collections.
+
+A hashref of options may also be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
+=back
+
+B<Warning:> if the number of collections is very large, this may return
+a very large result.  Either pass an appropriate filter, or use
+L</list_collections> to iterate over collections instead.
 
 =head2 get_collection, coll
 
@@ -562,20 +792,21 @@ L<MongoDB::Collection> constructor.
 
 The C<coll> method is an alias for C<get_collection>.
 
-=head2 get_gridfs
+=head2 get_gridfsbucket, gfs
 
-    my $grid = $database->get_gridfs;
-    my $grid = $database->get_gridfs("fs");
-    my $grid = $database->get_gridfs("fs", $options);
+    my $grid = $database->get_gridfsbucket;
+    my $grid = $database->get_gridfsbucket($options);
+    my $grid = $database->gfs($options);
 
-Returns a L<MongoDB::GridFS> for storing and retrieving files from the database.
-Default prefix is "fs", making C<$grid-E<gt>files> "fs.files" and C<$grid-E<gt>chunks>
-"fs.chunks".
+This method returns a L<MongoDB::GridFSBucket> object for storing and
+retrieving files from the database.
 
 It takes an optional hash reference of options that are passed to the
-L<MongoDB::GridFS> constructor.
+L<MongoDB::GridFSBucket> constructor.
 
-See L<MongoDB::GridFS> for more information.
+See L<MongoDB::GridFSBucket> for more information.
+
+The C<gfs> method is an alias for C<get_gridfsbucket>.
 
 =head2 drop
 
@@ -583,6 +814,18 @@ See L<MongoDB::GridFS> for more information.
 
 Deletes the database.
 
+A hashref of options may also be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
+=back
+
 =head2 run_command
 
     my $output = $database->run_command([ some_command => 1 ]);
@@ -592,17 +835,35 @@ Deletes the database.
         { mode => 'secondaryPreferred' }
     );
 
+    my $output = $database->run_command(
+        [ some_command => 1 ],
+        $read_preference,
+        $options
+    );
+
 This method runs a database command.  The first argument must be a document
 with the command and its arguments.  It should be given as an array reference
 of key-value pairs or a L<Tie::IxHash> object with the command name as the
-first key.  The use of a hash reference will only reliably work for commands
-without additional parameters.
+first key.  An error will be thrown if the command is not an
+L<ordered document|MongoDB::Collection/Ordered document>.
 
 By default, commands are run with a read preference of 'primary'.  An optional
 second argument may specify an alternative read preference.  If given, it must
 be a L<MongoDB::ReadPreference> object or a hash reference that can be used to
 construct one.
 
+A hashref of options may also be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
+=back
+
 It returns the output of the command (a hash reference) on success or throws a
 L<MongoDB::DatabaseError|MongoDB::Error/MongoDB::DatabaseError> exception if
 the command fails.
@@ -615,21 +876,60 @@ There are a few examples of database commands in the
 L<MongoDB::Examples/"DATABASE COMMANDS"> section.  See also core documentation
 on database commands: L<http://dochub.mongodb.org/core/commands>.
 
+=head2 aggregate
+
+Runs a query using the MongoDB 3.6+ aggregation framework and returns a
+L<MongoDB::QueryResult> object.
+
+The first argument must be an array-ref of L<aggregation
+pipeline|http://docs.mongodb.org/manual/core/aggregation-pipeline/> documents.
+Each pipeline document must be a hash reference.
+
+The server supports several collection-less aggregation source stages like
+C<$currentOp> and C<$listLocalSessions>.
+
+    $result = $database->aggregate( [
+        {
+            "\$currentOp" => {
+                allUsers => true,
+            },
+        },
+    ] );
+
+See L<Aggregation|http://docs.mongodb.org/manual/aggregation/> in the MongoDB manual
+for more information on how to construct aggregation queries.
+
+=head2 watch
+
+Watches for changes on this database.
+
+Perform an aggregation with an implicit initial C<$changeStream> stage
+and returns a L<MongoDB::ChangeStream> result which can be used to
+iterate over the changes in the database. This functionality is
+available since MongoDB 4.0.
+
+    my $stream = $db->watch();
+    my $stream = $db->watch( \@pipeline );
+    my $stream = $db->watch( \@pipeline, \%options );
+
+    while (1) {
+
+        # This inner loop will only run until no more changes are
+        # available.
+        while (my $change = $stream->next) {
+            # process $change
+        }
+    }
+
+The returned stream will not block forever waiting for changes. If you
+want to respond to changes over a longer time use C<maxAwaitTimeMS> and
+regularly call C<next> in a loop.
+
+See L<MongoDB::Collection/watch> for details on usage and available
+options.
+
 =for Pod::Coverage last_error
 
-=head1 DEPRECATIONS
-
-The methods still exist, but are no longer documented.  In a future version
-they will warn when used, then will eventually be removed.
-
-=over 4
-
-=item *
-
-last_error
-
-=back
-
 =head1 AUTHORS
 
 =over 4
@@ -640,6 +940,10 @@ David Golden <david@mongodb.com>
 
 =item *
 
+Rassi <rassi@mongodb.com>
+
+=item *
+
 Mike Friedman <friedo@friedo.com>
 
 =item *
@@ -654,7 +958,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc..
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
