@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,7 +19,7 @@ package MongoDB::Op::_FindAndUpdate;
 # Encapsulate find_and_update operation; atomically update and return doc
 
 use version;
-our $VERSION = 'v1.8.1';
+our $VERSION = 'v2.2.0';
 
 use Moo;
 
@@ -30,8 +28,9 @@ use MongoDB::Op::_Command;
 use Types::Standard qw(
     HashRef
 );
-use Try::Tiny;
-
+use MongoDB::_Types qw(
+    Boolish
+);
 use namespace::clean;
 
 has filter => (
@@ -43,7 +42,6 @@ has filter => (
 has modifier => (
     is       => 'ro',
     required => 1,
-    isa      => HashRef,
 );
 
 has options => (
@@ -52,11 +50,18 @@ has options => (
     isa      => HashRef,
 );
 
+has is_replace => (
+    is       => 'ro',
+    required => 1,
+    isa      => Boolish,
+);
+
 with $_ for qw(
   MongoDB::Role::_PrivateConstructor
   MongoDB::Role::_CollectionOp
   MongoDB::Role::_WriteOp
   MongoDB::Role::_BypassValidation
+  MongoDB::Role::_UpdatePreEncoder
 );
 
 sub execute {
@@ -67,14 +72,18 @@ sub execute {
             "MongoDB host '" . $link->address . "' doesn't support collation" );
     }
 
-    my ( undef, $command ) = $self->_maybe_bypass(
-        $link,
+    my $command = $self->_maybe_bypass(
+        $link->supports_document_validation,
         [
             findAndModify => $self->coll_name,
             query         => $self->filter,
-            update        => $self->modifier,
+            update        => $self->_pre_encode_update(
+                $link->max_bson_object_size,
+                $self->modifier,
+                $self->is_replace,
+            ),
             (
-                $link->accepts_wire_version(4)
+                $link->supports_find_modify_write_concern
                 ? ( @{ $self->write_concern->as_args } )
                 : ()
             ),
@@ -83,22 +92,28 @@ sub execute {
     );
 
     my $op = MongoDB::Op::_Command->_new(
-        db_name         => $self->db_name,
-        query           => $command,
-        query_flags     => {},
-        bson_codec      => $self->bson_codec,
+        db_name             => $self->db_name,
+        query               => $command,
+        query_flags         => {},
+        bson_codec          => $self->bson_codec,
+        session             => $self->session,
+        retryable_write     => $self->retryable_write,
+        monitoring_callback => $self->monitoring_callback,
     );
 
+    # XXX more special error handling that will be a problem for
+    # command monitoring
     my $result;
-    try {
+    eval {
         $result = $op->execute( $link, $topology );
         $result = $result->{output};
-    }
-    catch {
-        die $_ unless $_ eq 'No matching object found';
+        1;
+    } or do {
+        my $error = $@ || "Unknown error";
+        die $error unless $error eq 'No matching object found';
     };
 
-    # findAndModify returns ok:1 even for write concern errors, so 
+    # findAndModify returns ok:1 even for write concern errors, so
     # we must check and throw explicitly
     if ( $result->{writeConcernError} ) {
         MongoDB::WriteConcernError->throw(
