@@ -1,5 +1,4 @@
-#
-#  Copyright 2015 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 use strict;
 use warnings;
@@ -21,21 +19,30 @@ package MongoDB::Role::_PrivateConstructor;
 # MongoDB interface for a private constructor
 
 use version;
-our $VERSION = 'v1.8.1';
-
-use Moo::Role;
+our $VERSION = 'v2.2.0';
 
 use MongoDB::_Constants;
+use Sub::Defer;
 
-use namespace::clean;
+use Moo::Role;
 
 # When assertions are enabled, the private constructor delegates to the
 # public one, which checks required/isa assertions.  When disabled,
 # the private constructor blesses args directly to the class for speed.
 BEGIN {
+  my $NO_ASSERT_CONSTRUCTOR = <<'HERE';
+    my %done;
+    sub _new {
+      my $class = shift;
+      undefer_sub($class->can(q{new})) and $done{$class}++
+        unless $done{$class};
+      return bless {@_}, $class
+    }
+HERE
+
   WITH_ASSERTS
   ? eval 'sub _new { my $class = shift; $class->new(@_) }' ## no critic
-  : eval 'sub _new { my $class = shift; return bless {@_}, $class }'; ## no critic
+  : eval $NO_ASSERT_CONSTRUCTOR; ## no critic
 }
 
 1;
